#ifndef LOGGER_H
#define LOGGER_H

#include <ostream>
#include <iostream>
#include <QString>

enum LogType {

	LogWarning,
	LogError,
	LogInfo,
	LogDebug
};


class Logger{

protected:
	std::ostream& _out;

public:

	Logger(std::ostream& out) :
		_out(out)
	{

	}

	Logger(const char* msg, std::ostream& out) :
		Logger(out)
	{
		_out << msg << ": ";

	}

	template <typename T>
	Logger& operator << (const T& msg){
		_out << msg;
		return *this;
	}

	Logger& operator << (const QString& msg){
		(*this) << msg.toLocal8Bit();
		return *this;
	}

	Logger& operator << (const QByteArray& arr){
		for(const QChar& c : arr){
			(*this) << c;
		}

		return *this;
	}

	Logger& operator << (const QChar& c){

		(*this) << c.toAscii();


		return *this;
	}


	template<typename T, template <typename ELEM> class CONT>
	Logger& operator << (const CONT<T> list){

		for(const T& item : list){
			(*this) << item << ", ";
		}


		return *this;
	}


	~Logger(){
		_out << std::endl;
	}

};

Logger sp_log(LogType type);
#endif // LOGGER_H
